;;; This file contains a command interface that allows partial string matches on commands

;;; Use (display-local [person] [list - message to person] [list - message to others at place] ) for action reports
;;; Add display-region for area messages?
;;; Add display-global for messages everyone sees?
;;;; Commands to consider adding:
;; assist        action        armor         auction       afk           alias
;; aliases       alignment     areas         advance       attack        autoexit
;; autosplit     autounhide    autosize      ansicolor     backstab      backwards
;; bash          bank          bid           beep          berserk       brew
;; buy           bug           brief         blind         butcher       bribe
;; cast          close         ctell         clan          channels      consider
;; compare       cwho          clanwho       count         clanset       clear
;; connect       color         collect       compact       configure
;; context       changelog     changes       climb         clantell      clanlist
;; convert       calculate     disarm        dice          donate
;; drink         date          ditch         dues          disconnect    dream
;; deeds         douse         disfavor      defend        equipment
;; empty         enter         eat           extinguish
;; experience    flee          follow        followers     finger        find
;; fighting      faqs          forage        fish          favor
;; group         grshow        groupshow     grtank        grhurt        grwho
;; groupwho      groupcast     gtell         grouptell     gcast         give
;; gather        gold          godlist       godshow       gwho          grudges
;; guild         guildshow     help          hide          history       hints
;; hold          heal          hp            hptank        hphurt        hpgroup
;; home          hardreturns   information   idle          ignore
;; idea          insult        introduction  ignite        items         invert
;; kill          kick          light         list          listen
;; lock          level         levels        learn         leap          lastlogin
;; lineinvert    maintain      motd          manual        memorize
;; mudlist       mudshow       mudstat       math          map           melee
;; nosummon      notify        noprompt      noedit        noleave       nukereply
;; nogcast       nomelee       nocombat      order         open          out
;; put           pick          practice      pray          prompts       pkok
;; ppk           palm          pose          quaff         remove
;; report        reply         rehash        read          rescue        rest
;; recite        recall        retreat       roster        rob           reset
;; renounce      restore       speak         save          sacrifice
;; score         set           sell          shout         sit
;; skills        sleep         slist         search        sneak         sneaking
;; socials       sort          split         spells        spam          stats
;; status        statistics    stand         steal         sunset        send
;; slay          stopfighting  settarget     tell          talk          throw
;; trip          title         train         time          turn          tips
;; typo          tank          target        truce         use
;; usecarried    useheld       unlock        unuse         unwear        unwield
;; uptime        unalias       unmaintain    ungroup       uname         unhide
;; userload      value         variables     version       wear          wield
;; whois         whoami        wakeup        warn          where         waypoints
;; wander        write         whisper       window        wimpy         weight
;; wstat         worldstat     walk          wizlist       yell          

(define commands-list
  (list
   (list ;; a
    ;; attack
    )
   (list ;; b
    )
   (list ;; c
    (cons "commands"      ;;dynamic command list generation, should be changed later
          (lambda (args)
            (for-each
             (lambda (part-list)
               (for-each
                (lambda (x) (display (car x)) (display "    ") (display #\tab))
                part-list))
             commands-list)
            (newline)))
      ;; close
    )
   (list ;; d
    (cons "down"
          (lambda (args) (ask player 'go '(down))))
    (cons "drop"
          (lambda (args) (ask player 'drop args))) )
   (list ;; e
    (cons "east"
          (lambda (args) (ask player 'go '(east))))
    ;; equip
    (cons "emote"
          (lambda (args) (ask player 'emote args)))
    (cons "exits"
          (lambda (args)
            (let ((neighbors (ask (ask player 'place) 'neighbors))
                  (exits (ask (ask player 'place) 'exits)))
              (cond ((null? exits) (display-local player
                                                  '("No visible exits")
                                                  '() ))
                    (else
                     (for-each (lambda (destination direction)
                                 (display-local player
                                                (list direction ":" (ask destination 'name))
                                                '() ))
                               neighbors exits)
                     #t)))))
    (cons "examine"
          (lambda (args) (apply-zork 'look (cons 'at args)))) )
   (list ;; f
    )
   (list ;; g
    (cons "go"
          (lambda (args) (ask player 'go args)))
    (cons "get"
          (lambda (args) (apply-zork 'take args)))
    (cons "glance"
          (lambda (args)
            (let ((person (keyword-lookup (ask (ask player 'place) 'things) args)))
              (cond ((null? person) (display-local player
                                                   '("That person isn't here")
                                                   '() ))
                    ((is-a (car person) 'person?)
                     (display-local player
                                    (cons "  " (ask (car person) 'condition))
                                    '() )
                     #t)
                    (else (display-local player
                                         '("   That's not a person")
                                         '() ) ))))) )
   (list ;; h
    )
   (list ;; i
    (cons "i"
          (lambda (args) (ask player 'inv)))
    (cons "in"
          (lambda (args) (ask player 'go '(in))))
    (cons "inventory"
          (lambda (args) (ask player 'inv))) )
   (list ;; j
    )
   (list ;; k
    ;; kill
    )
   (list ;; l
    (cons "look"
          (lambda (args)
            (cond ((not (pair? args))
                   (display-local player
                                  (list (ask (ask player 'place) 'name))
                                  '() )
                   (display-local player
                                  (list "  " (ask (ask player 'place) 'long-desc))
                                  '() )
                   (display-local player
                                  (cons "Visible exits: " (ask (ask player 'place) 'exits))
                                  '() )
                   (let ((other-things (delq player (ask (ask player 'place) 'things))))
                     (if (not (null? other-things))
                              (display-local player
                                             (cons "   You see"  (pretty-object-list other-things))
                                             '() ))))
                  ((eq? (car args) 'at)
                   (let ((thing (keyword-lookup (ask (ask player 'place) 'things) (cdr args) )))
                     (if (null? thing)
                         (begin
                           (display-message (list "You don't see that"))
                           #f)
                         (begin
                           (if (eq? (ask (car thing) 'long-desc) 'blank)
                               (display-message (list "  " (ask (car thing) 'short-desc)))
                               (display-message (list "  " (ask (car thing) 'long-desc))))
                           (if (is-a (car thing) 'person?)
                               (display-local player
                                              (cons "  " (ask (car thing) 'condition))
                                              '() ))
                           #t))))
                  (else (apply-zork 'look '()) )))) )
   (list
    (cons "move"
          (lambda (args) (ask player 'go args))) )
   (list ;; n
    (cons "north"
          (lambda (args) (ask player 'go '(north))))
    (cons "northeast"
          (lambda (args) (ask player 'go '(northeast))))
    (cons "northwest"
          (lambda (args) (ask player 'go '(northwest))))
    (cons "ne"
          (lambda (args) (ask player 'go '(northeast))))
    (cons "nw"
          (lambda (args) (ask player 'go '(northwest)))) )
   (list ;; o
     ;; open
    (cons "out"
          (lambda (args) (ask player 'go '(out)))) )
   (list ;; p
    )
   (list ;; q
    (cons "quit"    ;; quit defined in interpreter to bypass partial matches
          (lambda (args) (display-message '("If you want to quit, type the whole word")))) )
   (list ;; r
    )
   (list ;; s
    (cons "south"
          (lambda (args) (ask player 'go '(south))))
    (cons "southeast"
          (lambda (args) (ask player 'go '(southeast))))
    (cons "southwest"
          (lambda (args) (ask player 'go '(southwest))))
    (cons "say"
          (lambda (args) (ask player 'say args)))
    (cons "status"
          (lambda (args)
            (display-local player
                           (list (ask player 'get-hp) "/" (ask player 'get-maxhp) " Hp")
                           '() )
            (display-local player
                           (ask player 'condition)
                           '() )
            (display-local player
                           '()
                           '() )))
    ;; slay
    (cons "scan"
          (lambda (args)
            (let ((neighbors (ask (ask player 'place) 'neighbors))
                  (exits (ask (ask player 'place) 'exits)))
              (for-each (lambda (destination direction)
                          (display direction) (display ": ")
                          (display-message (map (lambda (person) (ask person 'name))
                                                (other-people-at-place player destination))))
                        neighbors exits)
              #t)))
    (cons "se"
          (lambda (args) (ask player 'go '(southeast))))
    (cons "sw"
          (lambda (args) (ask player 'go '(southwest)))) )
   (list ;; t
    (cons "take"
          (lambda (args) (ask player 'take args))) )
   (list ;; u
    (cons "up"
          (lambda (args) (ask player 'go '(up)))) )
   (list ;; v
    )
   (list ;; w
    (cons "west"
          (lambda (args) (ask player 'go '(west))))
    (cons "wait"
          (lambda (args) #t)) )  ;; take no action
     ;; wear
     ;; wield
   (list ;; x
    )
   (list ;; y
    )
   (list ;; z
    (cons "z"
          (lambda (args) #t)) )  ;; shorthand for wait
    )  )

;;; --------------------------------------------------------------------------
;;; And here's the interpreter for ZORK-style commands...

;; *must* define 'player' somewhere, as a person

;; Input is plain text with no parens! Woo!
(define (run-game)
  (display (ask player 'get-hp))
  (display "/")
  (display (ask player 'get-maxhp))
  (display "Hp ")
  (display (ask (ask player 'place) 'name))
  (display "> ")
  (let ((input (get-input)))
    (cond ((null? input) (newline) (run-game))
          ((eq? (car input) 'quit)
           'done)
           (else
            (apply-zork (car input)(cdr input))
            (clock)
            (run-game)))))

;; find first command in table matching asymbol and return it
(define (lookup-command asymbol)
  (let ((part (symbol->string asymbol)))
     (let ((possibles (list-ref commands-list (- (char->integer (char-downcase (string-ref part 0)))
                                                 (char->integer #\a)))))
;       (display-message (list "Possibles:" (map car possibles)))       ;;DEBUGGING MESSAGE
        (let ((whats-left (filter (lambda (x)
                                    (if (< (string-length (car x))      ;; too short
                                           (string-length part))
                                        #f
                                        (string-ci=? (substring (car x) 0 (string-length part))
                                                    part)))
                                  possibles)))                   ;; does it match up?
;          (display-message (list "Remaining:" (map car whats-left)))   ;;DEBUGGING MESSAGE
           (if (null? whats-left)
               '()
               (cdar whats-left))))))     ;; first proc with string that matches

(define (get-input)
  (let ((x (read)))
    (if (eqv? (peek-char) #\newline)
        (cons x '() )
        (cons x (get-input) ) )))


(define (apply-zork request args)
  (let ((method (lookup-command request)))
    (if (null? method)
        (display-message (list "Command" request "not implemented"))
        (method args))))

;; shows (car strings) to acting person; shows (cadr strings), if not null, to others at place
(define (display-local person . messages)
  (cond ((eq? person player)
         (if (not (null? (car messages)))
             (display-message (car messages))))
        ((and (not (null? (cadr messages))) (eq? (ask person 'place)(ask player 'place)))
         (display-message (cadr messages)))
        (else 'do-nothing)))
